@@ -66,6 +66,8 @@ DllMain (HINSTANCE hinstDLL,
 
 #ifdef __APPLE__
 #import <Foundation/Foundation.h>
+#include <dlfcn.h>
+#include <libgen.h>
 #endif
 
 #include "gegl-debug.h"
@@ -254,16 +256,50 @@ gegl_init_get_prefix (void)
 
       if (exe)
         {
-          char *sep1, *sep2;
+          char   *sep1, *sep2;
+          size_t  len;
 
-          sep1 = strrchr (exe, G_DIR_SEPARATOR);
+          len   = strlen (exe);
+          sep1  = strrchr (exe, G_DIR_SEPARATOR);
           *sep1 = '\0';
 
           sep2 = strrchr (exe, G_DIR_SEPARATOR);
-          if (sep2 != NULL && g_strcmp0 (sep2 + 1, "bin") == 0)
+          if (sep2 != NULL)
             {
-              *sep2 = '\0';
-              prefix = exe;
+              while (g_strcmp0 (sep2 + 1, "bin") != 0 &&
+                     strstr (sep2 + 1, "lib") != sep2 + 1)
+                {
+                  *sep2 = '\0';
+                  sep2  = strrchr (exe, G_DIR_SEPARATOR);
+
+                  if (sep2 == NULL)
+                    break;
+                }
+
+              if (sep2 == NULL)
+                {
+                  while (strlen (exe) < len)
+                    exe[strlen (exe)] = G_DIR_SEPARATOR;
+
+                  /* This may happen when GEGL is loaded by uninstalled
+                   * binaries, such as build-time tools, in which case, this is
+                   * not an error, and we fallback to the build-time prefix.
+                   * We assume that relocatable builds are not relocated during
+                   * the build of a full bundle.
+                   * This is why we output a message on stderr, but this is not
+                   * a fatal error.
+                   */
+                  fprintf (stderr,
+                           "Relocatable builds require the executable to be installed in bin/ or lib*/ unlike: %s\n"
+                           "If this is a build-time tool, you may ignore this message.\n",
+                           exe);
+                  g_free (exe);
+                }
+              else
+                {
+                  *sep2 = '\0';
+                  prefix = exe;
+                }
             }
           else
             {
@@ -674,7 +710,28 @@ gegl_get_default_module_paths(void)
     }
 
   /* System library dir */
-#if defined(G_OS_WIN32)
+#if defined (__APPLE__)
+  Dl_info info;
+  /* The checked symbol must be exported in the libgegl*.dylib */
+  if (dladdr((const void *)gegl_get_default_module_paths, &info) && info.dli_fname)
+    {
+      char  dylib_path[PATH_MAX];
+      char *dylib_dir;
+  
+      /* Get the parent directory containing the .dylib (e.g. <foobar>\Frameworks\,
+         does not matter the parent dir, this is packaging-agnostic) */
+      strlcpy(dylib_path, info.dli_fname, sizeof(dylib_path));
+      dylib_dir = dirname(dylib_path);
+          
+      /* Construct the gegl dir on the parent dir (e.g. <foobar>\Frameworks\{GEGL_LIBRARY}) */
+      module_path = g_build_filename(dylib_dir, GEGL_LIBRARY, NULL);
+    }
+  else
+    {
+      g_critical ("Getting module path for relocatibility failed\n");
+      module_path = g_build_filename (LIBDIR, GEGL_LIBRARY, NULL);
+    }
+#elif defined(G_OS_WIN32)
   prefix      = g_win32_get_package_installation_directory_of_module (hLibGeglModule);
   module_path = g_build_filename (prefix, "lib", GEGL_LIBRARY, NULL);
   g_free(prefix);
